
#include <propeller.h>
#include <Bresenham.h>

void Bresenham::Init(void)
{
    _OUTA &= ~(s0 | d0 | s1 | d1 | s2 | d2);
    _DIRA |=  (s0 | d0 | s1 | d1 | s2 | d2);
}

void Bresenham::SetupMove( int DT, int A, int B, int C )
{
    dt = DT;
    i = dt;

    da = A;
    db = B;
    dc = C;
    era = dt >> 1;
    erb = era;
    erc = era;

    if( da < 0 ) {
        OUTA |= d0;
        da = -da;
    }
    else OUTA &= ~d0;

    if( db < 0 ) {
        OUTA |= d1;
        db = -db;
    }
    else OUTA &= ~d1;

    if( dc < 0 ) {
        OUTA |= d2;
        dc = -dc;
    }
    else OUTA &= ~d2;
}

void Bresenham::Tick(void)
{
    if( i == 0 ) return;

    era -= da;
    erb -= db;
    erc -= dc;

    if( era < 0 ) {
        era += dt;
        OUTA |= s0;
    }

    if( erb < 0 ) {
        erb += dt;
        OUTA |= s1;
    }

    if( erc < 0 ) {
        erc += dt;
        OUTA |= s2;
    }
    i--;

    waitcnt( 100 + CNT );
    OUTA &= ~(s0 | s1 | s2);
}
